// matrizen.cc 
#include <iostream>    // notwendig zur Ausgabe
#include "hdnum.hh"    // hdnum header
using namespace hdnum; // Namen ohne hdnum:: verwenden 

// Beispiel wie man A und b fr ein
// Gleichungssystem initialisieren knnte
template<class T>
void initialize (Matrix<T> A, Vector<T> b)
{
  if (A.rowsize()!=A.colsize() || A.rowsize()==0) 
    HDNUM_ERROR("need square and nonempty matrix");
  if (A.rowsize()!=b.size())
    HDNUM_ERROR("b must have same size as A");
  for (int i=0; i<A.rowsize(); ++i)
	{
	  b[i] = 1.0;
      for (int j=0; j<A.colsize(); ++j)
		if (j<=i) A[i][j]=1.0; else A[i][j]=0.0;
	}
}

int main ()
{
  // Konstruktion
  Matrix<float> A;            // leere Matrix mit Gre 0x0
  Matrix<float> B(10,10);     // 10x10 Matrix uninitialisiert
  Matrix<float> C(10,10,0.0); // 10x10 Matrix initialisiert

  // Zugriff auf Vektorelemente
  for (int i=0; i<A.rowsize(); ++i)
    for (int j=0; j<A.colsize(); ++j)
      B[i][j] = 0.0;          // jetzt ist B initialisiert

  // Kopie und Zuweisung
  Matrix<float> D(B);         // D identisch mit B! Keine Kopie
  A = D;                      // A ist nun identisch mit D!
  A[0][0];                    // ndert auch B[0][0], D[0][0]

  Matrix<float> E(copy(B));   // E ist echte Kopie
  A = copy(B);                // und auch A

  Matrix<float> F(A.sub(1,2,3,4));// F ist 3x4 Teilmatrix ab (1,2)

  // Rechnen mit Matrizen und Vektoren
  A += B;                     // A = A+B
  A -= B;                     // A = A-B
  A *= 1.23;                  // skalare Multiplikation
  A /= 1.23;                  // skalare Division
  A.update(1.23,B);           // A = A + s*B

  Vector<float> x(10,1.0);    // make two vectors
  Vector<float> y(10,2.0);
  A.mv(y,x);                  // y = A*x
  A.umv(y,x);                 // y = y + A*x
  A.umv(y,(float)-1.0,x);     // y = y + s*A*x
  C.mm(A,B);                  // C = A*B
  C.umm(A,B);                 // C = C + A*B

  // Ausgabe
  std::cout << A.sub(0,0,3,3) << std::endl;// schne Ausgabe
  A.iwidth(2);                // Stellen in Indexausgabe
  A.width(11);                // Anzahl Stellen gesamt
  A.precision(4);             // Anzahl Nachkommastellen
  std::cout << A << std::endl;// nun mit mehr Stellen

  // Hilfsfunktionen
  identity(A);                // setze A auf Einheitsmatrix
  std::cout << A << std::endl;
  spd(A.sub(6,6,4,4));        // eine s.p.d. Matrix
  std::cout << A << std::endl;
  std::cout << A.sub(6,6,4,4) << std::endl;
  fill(x,(float)1,(float)1);
  vandermonde(A,x);           // Vandermondematrix
  std::cout << A << std::endl;

  // Funktionsaufruf
}
