// vektoren.cc 
#include <iostream>    // notwendig zur Ausgabe
#include "hdnum.hh"    // hdnum header
using namespace hdnum; // Namen ohne hdnum:: verwenden 

template<class T>
void product (Vector<T> x)     // x ist keine echte Kopie!
{
  for (int i=1; i<x.size(); i=i+1)
    x[i] = x[i]*x[i-1];
}

template<class T>
T sum (Vector<T> x)     // x ist keine echte Kopie!
{
  T s(0.0);
  for (int i=0; i<x.size(); i=i+1)
	s = s + x[i];
  return s;
}

int main ()
{
  // Konstruktion
  Vector<float> x(10);        // Vektor mit 10 Elementen
  Vector<double> y(10,3.14);  // 10 Elemente initialisiert
  Vector<float> a;            // ein leerer Vektor
  x.resize(117);              // vergrern, Daten gelscht!
  x.resize(23,2.71);          // verkleinern geht auch

  // Zugriff auf Vektorelemente
  for (int i=0; i<x.size(); i=i+1)
	x[i] = i;                 // Zugriff auf Elemente 

  // Kopie und Zuweisung
  Vector<float> z(x);         // Kopie ist eine Referenz auf gleiche Daten
  z[2] = 1.24;                // hat den gleichen Effekt wie x[2] = 1.24 !

  a = z;                      // a referenziert die Daten von z
  a[2] = -0.33;               // ndert auch z[2] und x[2];
  a = 5.4;                    // Zuweisung an alle Elemente
  
  Vector<float> b(copy(a));   // b ist echte Kopie von a,x,z
  a = copy(x);                // a hlt echte Kopie der Daten von x

  Vector<float> w(x.sub(8,4));// w referenziert x[8],...,x[11] 
  z = x.sub(3,4);             // z referenziert x[3],...,x[6]

  // Rechnen mit Vektoren
  w += z;                     // w = w+z
  w -= z;                     // w = w-z
  w *= 1.23;                  // skalare Multiplikation
  w /= 1.23;                  // skalare Division
  w.update(1.23,z);           // w = w + a*z
  x[0] = w*z;                 // skalare Multiplikation

  // Ausgabe
  std::cout << w << std::endl;// schne Ausgabe
  w.iwidth(2);                // Stellen in Indexausgabe
  w.width(20);                // Anzahl Stellen gesamt
  w.precision(16);            // Anzahl Nachkommastellen
  std::cout << w << std::endl;// nun mit mehr Stellen

  // Hilfsfunktionen
  zero(w);                    // das selbe wie w=0.0
  fill(w,(float)1.0);         // das selbe wie w=1.0
  fill(w,(float)0.0,(float)0.1); // w[0]=0, w[1]=0.1, w[2]=0.2, ...
  unitvector(w,2);            // kartesischer Einheitsvektor
  gnuplot("test.dat",w);      // gnuplot Ausgabe: i w[i]
  gnuplot("test2.dat",w,z);   // gnuplot Ausgabe: w[i] z[i]

  // Funktionsaufruf
  product(x);
  std::cout << sum(x) << std::endl;
}
